/// <reference path="../Definitions/jquery.d.ts"/>
/// <reference path="../Definitions/knockout.d.ts"/>
/// <reference path="../Definitions/knockout.mapping.d.ts"/>
/// <reference path="../Definitions/SharePoint.d.ts"/>

// Interface
interface JQuery {
    /** wookmark is the plugin that does picture layout.  
     * TypeScript doesn't know what it is, so I defined the method that I'll be using.
     * like all good JQuery methods, it returns the JQuery object. */
    wookmark: (any) => JQuery;
}

module utility {
    /** fixes MS.AJAX's funny date to a javascript date object */
    export function fixDate(value: string): Date {
        var rd = new RegExp("Date\\([\\d+-]+\\)");
        var m = rd.exec(value);
        if (m) {
            return <Date> eval("new " + m[0]);
        }
    }

    /** splits a picture path to find the thumbnail image url */
    export function getThumbnailUrl(url: string): string {
        // http://spg-dev-jl/PublishingImages/Photo%20(2).jpg
        // http://spg-dev-jl/PublishingImages/_w/Photo%20(2)_jpg.jpg
        var re = /(.*)\/([^\/]*)\.(.*)/gi;
        var m = re.exec(url);
        if (m) {
            return m[1] + "/_w/" + m[2] + "_" + m[3] + ".jpg";
        }
        return url;
    }

    /** splits a SPFieldLink into two arguments */
    export function cleanSPFieldLink(link: string): string {
        var re = /^(.*), /gi;
        var m = re.exec(link);
        if (m) return m[1];
        return link;
    }
}

// Module
module PInteresp {

    // Class
    export class PictureItemViewModel {

        // after TS 0.9, this line will be:
        // public name : KnockoutObservable<string> = ko.observable("");
        public name = ko.observable("");
        public name1 = "";
        // public width : KnockoutObservable<number> = ko.observable(0);
        public width = ko.observable(0);
        public height = ko.observable(0);
        public createdBy = ko.observable("");
        public createdByPicture = ko.observable("");
        public modifiedOn: KnockoutObservable<Date> = ko.observable(null);
        public src = ko.observable("");
        public srcComputed: KnockoutComputed<string>;

        constructor(data: any) {

            this.name(data.Name);
            this.width(data.Width);
            this.height(data.Height);
            this.createdBy(data.CreatedBy ? data.CreatedBy.Name : "");
            this.createdByPicture(data.CreatedBy && data.CreatedBy.Picture ? utility.cleanSPFieldLink(data.CreatedBy.Picture) : "/_layouts/images/O14_person_placeHolder_96.png");
            this.modifiedOn(utility.fixDate(data.Modified));
            this.src(data.__metadata.media_src);

            this.srcComputed = ko.computed(() => {
                return utility.getThumbnailUrl(this.src());
            });
        }

        public view() {
            SP.UI.ModalDialog.showModalDialog({
                url: this.src(),
                title: this.name()
            });
        }

        public getUser() {
            alert(this.createdBy());
        }

        public like() {
            // not implmented yet!
        }

    }

    // rename symbol
    export class PicturesLibraryViewModel {

        // knockout uses a mapping object when cloning child objects
        private mapping: KnockoutMappingOptions = {
            create: function (options) {
                return new PictureItemViewModel(options.data);
            }
        }

        private items: KnockoutObservableArray<PictureItemViewModel>;

        constructor() {
            this.items = ko.mapping.fromJS([], this.mapping);
        }

        /**
        * call rest interface and load the viewmodel with a set of pictures
        */
        public load() {
            var self = this;
            var restUrl = _spPageContextInfo.webServerRelativeUrl + 
                "_vti_bin/listdata.svc/Images/?$expand=CreatedBy&$select=Name,Width,Height,CreatedBy,Modified";

            $.ajax({
                type: "GET",
                url: restUrl,
                contentType: "application/json; charset=utf-8",
                dataType: "json",
                cache: false,
                processData: true,
                success: function (data , status, xhr) {

                    if (status == "success" && data) {
                        // using knockout mapping to map json objects into 
                        // knockout observable array
                        ko.mapping.fromJS(data.d.results, self.mapping, self.items)
                    }
                },
                error: function (xhr, status, err) {
                    alert(err);
                }
            });
        }

        public wookmark() {
            $('div.pictures div.listitem').wookmark({ offset: 15, container: $('div.pictures'), autoResize: true });
        }
    }
}

var vm = new PInteresp.PicturesLibraryViewModel();
$(document).ready(function () {
    ko.applyBindings(vm);
    vm.load();
});

// toggle outline expansion
// checkout sourcemap