﻿


module tools {
    export function findReturnType(o: any): string {

        // return; 
        if (/return\s*;/gi.test(o)) return ": void";

        // no return 
        if (!/return/gi.test(o)) return ": void";

        // return 1.1; return 1; 
        if (/return\s*\d+(\.\d+)?;/gi.test(o)) return ": number";

        // return "xx"; 
        if (/return\s*"[^"]*?"\s*;/gi.test(o)) return ": string";

        // return 'xx'; 
        if (/return\s*'[^']*?'\s*;/gi.test(o)) return ": string";

        // return something; 
        if (/return\s*[^;]+?;/gi.test(o)) return ": any";

        // don't know 
        return ": any";
    }

    export function toClass(o: any): string {
        var isClass = o["__class"] == true;
        var typeName = <string> o["__typeName"];

        typeName = typeName.replace(/\./gi, "_");

        var results = "interface " + typeName + "Static {\n";
        var m, member, match;

        for (member in o) {

            if (member.match(/\$|__/)) continue;

            m = o[member];
            if (typeof (m) == "function") {
                match = /function\s*(\(.*?\))/gi.exec(m);
                if (match) {
                    results += "" + member + match[1] + findReturnType(m) + ";\n";
                }
            }
            else {
                results += "" + member + " : " + typeof (m) + ";\n";
            }
        }

        results += "}\n";

        results += "interface " + typeName + " {\n";

        for (member in o.prototype) {

            if (member.match(/\$|__/)) continue;

            m = o.prototype[member];
            if (typeof (m) == "function") {
                match = /function\s*(\(.*?\))/gi.exec(m);
                if (match) {
                    results += "" + member + match[1] + findReturnType(m) + ";\n";
                }
            }
            else {
                results += "" + member + " : " + typeof (m) + ";\n";
            }
        }

        results += "}\n";

        return results;

    }

    export function toEnum(o: any): string {

        var typeName = <string> o["__typeName"];
        typeName = typeName.replace(/\./gi, "_");
        var results = "interface " + typeName + "{\n";

        for (var member in o) {

            if (member.match(/\$|__/)) continue;

            var m = o[member];
            if (typeof (m) == "function") {
                continue;
            }
            else {
                results += "" + member + " : " + typeof (m) + ";\n";
            }
        }

        results += "}\n";
        return results;
    }

    export function toTypeScript(o: any): string {
        var isClass = o["__class"] == true;
        var isNamespace = o["__namespace"] == true;
        var isEnum = o["__enum"] == true;

        if (isClass) {
            return toClass(o);
        }
        if (isEnum) {
            return toEnum(o);
        }
        return "";
    }
}